//======================================================= file = demux.c ======
//=  Demultiplexor for IYER PPS (centralized) switch simulation model         =
//=============================================================================
//=  Contains: demux()
//=---------------------------------------------------------------------------=
//=  History:  Ahmed (04/01/03) - Genesis
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include <math.h>       // Needed for ceil()
#include <assert.h>     // Needed for assert() macro
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff
//=============================================================================

void demux(int in_id,int out_id,double org_time)
{
  int      channel_num;   // Internal switch plane #, to dispatch a cell 
  int      i,j,r;         // Generic loop counters
  int      flag1;         // Boolean flag

  create("demux");

  // Checking for the free internal inport links
  for(i=0;i<K;i++)
  {
    if((clock>=Ail_freetime[i][in_id]) && (Ail[i][in_id]==FALSE))
    {
      Ail[i][in_id] = TRUE;
    }
    if((Freeslot[out_id]>=Aol_freetime[i][out_id]) && (Aol[i][out_id]==FALSE))
    {
      Aol[i][out_id] = TRUE;
    }
  }

  // Selecting a free, least recently used internal input link (by flow)
  flag1 = FALSE;
  j = Out_seq[in_id][out_id];
  for(i=0;i<K;i++)
  {
    r = (j+i)%K;
    if((Ail[r][in_id] == TRUE) && (Aol[r][out_id] == TRUE))
    {
      channel_num = r;
      Out_seq[in_id][out_id] = (r+1)%K;
      Ail[r][in_id]  = FALSE;
      Aol[r][out_id] = FALSE;
      Ail_freetime[r][in_id]= clock + ceil(K/S);
      Freeslot[out_id] = Freeslot[out_id] + 1;
      Aol_freetime[r][out_id] = Freeslot[out_id] + ceil(K/S);
      flag1 = TRUE;
      break;
    }
  }

  // Assert, If all internal input links are busy
  if(flag1 == FALSE)
  {
    assert(FALSE);
  }
  else
  {
    assert(channel_num<K);
    // Send cell to an internal channel queue at demux
    chan[channel_num][in_id](in_id,out_id,channel_num,
                             Freeslot[out_id],org_time);
  }
}

//=============================================================================
//==  Function for an internal input channel at demux                        ==
//=============================================================================
void channel(int in_id,int out_id,int channel_num,int tag, double org_time)
{
  create("channel");

  // if Queue is full, Drop the cell
  if(qlength(Channel_server[channel_num][in_id])>0)
  {
    Demux_loss[in_id]++;
    return;
  }

  // Queue the cell on the channel queue
  reserve(Channel_server[channel_num][in_id]);
  hold(ceil(K/S)*CELL_TIME);
  release(Channel_server[channel_num][in_id]);

  // Forward cell to the inport of the selected switch plane
  in[channel_num][in_id](in_id,out_id,channel_num,tag,org_time);
  return;
}
