//========================================================= file = mux.c ======
//=  Outport multiplexor for IYER PPS (centralized) switch simulation model   =
//=============================================================================
//=  Contains: mux()
//=---------------------------------------------------------------------------=
//=  History: AAS (04/01/03) - Genesis                                        =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

void mux(int from_id,int out_id,int tag,double org_time)
{
  create("mux");

  // Drop the cell, if Queue is full
  if(qlength(Outport_external[out_id])>0)
  {
    Mux_loss[out_id]++;
    return;
  }

  // Queue for transmitting cells
  reserve(Outport_external[out_id]);
  hold(CELL_TIME);
  release(Outport_external[out_id]);

  // Record delay statistics for cells leaving this port
  record((clock-org_time),Delay_table[out_id]);
  Outport_count[out_id]++;

  // Recording re-ordering of cells per outport
  if(out_id==0)
  {
    Count++;
    if(tag==Index)
    {
      Cell_served[Index] = TRUE;
      Index++;
      while(Cell_served[Index] == TRUE)
      {
        Index++;
      }
    }
    else
    {
      Cell_served[tag] = TRUE;
      Reorder++;
    }
  }
  return;
}
