//============================================== file = mux_sel.c =============
//=  Mux re-assembly algorithm for IYER PPS (centralized) switch model        =
//=============================================================================
//=  Contains: mux_sel(int out_id)                                            =
//=---------------------------------------------------------------------------=
//=  History: AAS (04/01/03) - Genesis                                        =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

void mux_sel(int out_id)
{

  int      i;       // Generic loop counter
  int      DT;      // Sequence # of next in-sequence cell
  long int dep_time; // Departure time of HOL cell in intenral outport queue 
  int      flag[K]; // Boolean flag   
  int      first_flag; // Boolean flag 
  int      temp1[K]; // Temporary varaibles
  int      minimum; // Minimum departure time of a cell among the HOL cells
  int      min_layer; // Plane # of a cell (having minimum departure time)

  create("mux_sel");

  // Initializing the variables
  for(i=0;i<K;i++)
  {
    flag[i] = FALSE;
    temp1[i]= 0;
  }
  DT = 1;

  // Infinite Loop
  while(TRUE)
  {
    for(i=0;i<K;i++)
    {
      // Receving  departure time of HOL cell from internal outports
      if((queue_cnt(Q_event[i][out_id])>0)&&(flag[i]==FALSE))
      {
        receive(Seq_number[i][out_id], (long *) &dep_time);
        temp1[i] = dep_time;
        flag[i] = TRUE;
      }
    }

    // Finding HOL cell with minimum departure time
    first_flag = TRUE;
    for(i=0;i<K;i++)
    {
      if((first_flag == TRUE)&&(flag[i]==TRUE))
      {
        minimum = temp1[i];
        min_layer = i;
        first_flag = FALSE;
      }
      else
      {
        if((flag[i]==TRUE)&&(temp1[i]<minimum))
        {
          minimum = temp1[i];
          min_layer = i;
        }
      }
    }

    // If the selected cell is next in sequence,transmit it
    if(first_flag==FALSE)
    {
      if(minimum == DT)
      {
        DT++;
        set(Q_event[min_layer][out_id]);
        flag[min_layer]= FALSE;
        hold(CELL_TIME);
      }
      // else wait for an empty cell slot
      else
      {
        hold(CELL_TIME);
      }
    }

    // If all outport buffers are empty, wait for a cell slot
    else
    {
      hold(CELL_TIME);
    }
  }
}
